from .dim4 import PcaModelSelect

trans = PcaModelSelect(score='aic', tol=1e-2)

meta = {
    'id': 'dim5',
    'name': 'PCA AIC',
    'description': (
        "Number of components is determined by fitting a Lasso-Logit "
        "model and minimize the AIC criteron."),
    'keywords': [
        'dimensionality reduction', 'principal component anlysis',
        'StandardScaler', 'PCA', 'AIC', 'Logistic Regression',
        'statsmodels'],
    'feature_names_prefix': 'dim_aic'
}

"""Example

from verto.dim5 import trans, meta
from datasets.demo1 import X_train, Y_train
from seasalt import create_feature_names
import pandas as pd
import numpy as np

#trans.set_params(**{'verbose': True, 'tol': 1e-2})
X_new = trans.fit_transform(X_train, Y_train)
names = create_feature_names(meta['feature_names_prefix'], X_new.shape[1])
df = pd.DataFrame(data=X_new, columns=names)
df
"""




from sklearn.base import BaseEstimator, TransformerMixin


class ClusterTransformer(BaseEstimator, TransformerMixin):
    def __init__(self, model):
        self.model = model

    def fit(self, *args, **kwargs):
        self.model.fit(*args, **kwargs)
        return self

    def transform(self, X, **transform_params):
        return self.model.predict(X).reshape(-1, 1)



import re
import itertools as it
import numpy as np
from seasalt import create_feature_names


def transformer_custom_settings(global_dict, X, y=None, fit=True, names=True):
    r = re.compile(r"trans\d+")
    Z = []
    f = []
    for st in list(filter(r.match, global_dict)):
        sidx = st[5:]
        sm = "meta" + sidx
        if sidx.isdigit():
            print(sidx, st, sm)
            if fit:
                global_dict[st].fit(X, y)
            Z_tmp = global_dict[st].transform(X)
            Z.append(Z_tmp)
            if names:
                f.append(create_feature_names(
                    global_dict[sm]['feature_names_prefix'], Z_tmp.shape[1]))

    return np.concatenate(Z, axis=1), list(it.chain.from_iterable(f))



from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler
from sklearn.cluster import MiniBatchKMeans
from sklearn.preprocessing import OneHotEncoder
from seasalt import ClusterTransformer
import numpy as np


class KMeansOnehot(Pipeline):
    def __init__(self, n_clusters=3):
        super().__init__(steps=[
            ('scl', StandardScaler()),  # with_mean=True,with_std=True
            ('kmeans', ClusterTransformer(MiniBatchKMeans(n_clusters = 3))),
            ('onehot', OneHotEncoder(sparse=False, categories='auto'))
        ])
        super().set_params(**{'kmeans__model__n_clusters': n_clusters})

    def set_params(self, **kwargs):
        if 'n_clusters' in kwargs:
            super().set_params(
                **{'kmeans__model__n_clusters': kwargs['n_clusters']})

    def transform(self, X, y=None):
        return super().transform(X).astype(np.int8)


trans = KMeansOnehot()

meta = {
    'id': 'clu1',
    'name': 'KMeans OneHot',
    'description': (
        "Onehot encoded KMeans clusters "),
    'keywords': [
        'KMeans', 'clustering', 'unsupervised',
        'StandardScaler', 'OneHotEncoder', 'MiniBatchKMeans'],
    'feature_names_prefix': 'clu_km'
}

"""Example

from verto.clu1 import trans, meta
from datasets.demo1 import X_train
from seasalt import create_feature_names
import pandas as pd
import numpy as np

trans.set_params(**{'n_clusters': 4})
X_new = trans.fit_transform(X_train)
names = create_feature_names(meta['feature_names_prefix'], X_new.shape[1])
df = pd.DataFrame(data=X_new, columns=names)
df
"""



from sklearn.ensemble import IsolationForest
from sklearn.base import BaseEstimator, TransformerMixin
import numpy as np


class IsolationForestTransformer(BaseEstimator, TransformerMixin):
    def __init__(self, model):
        self.model = model

    def fit(self, *args, **kwargs):
        self.model.fit(*args, **kwargs)
        return self

    def transform(self, X, **transform_params):
        return self.model.score_samples(X).astype(np.float16).reshape(-1, 1)


trans_base = IsolationForest(
    n_estimators = 96,  # number of base trees
    contamination = 0.1,  # pct of outliers = pct of TRUE targets
    max_samples = 'auto',  # to train 1 base tree; 'auto'=max(256, n_samples)
    max_features = 1.0,   # use all features
    # other
    bootstrap = False,  # False: without replacement; Don't reuse
    random_state = 42,
    behaviour = 'new',
    verbose = False,
    n_jobs = -1
)

trans = IsolationForestTransformer(trans_base)

meta = {
    'id': 'out1',
    'name': 'Iso. Forest',
    'description': (
        "Isolation Forest partitions samples randomly. The path length "
        "(or number of splits) is short for anomalies (anomal samples)"),
    'keywords': [
        'IsolationForest', 'Isolation Forest', "outlier detection"],
    'feature_names_prefix': 'out_iso'
}

"""Example

from verto.out1 import trans, meta
from datasets.demo1 import X_train, Y_train
from seasalt import create_feature_names
import pandas as pd
import numpy as np

trans.set_params(**{'model__contamination': 0.2})
X_new = trans.fit_transform(X_train)
names = create_feature_names(meta['feature_names_prefix'], X_new.shape[1])
df = pd.DataFrame(data=X_new, columns=names)
df
"""



from .cluster_transformer import ClusterTransformer
from .store_models import to_disk, from_disk
from .refit_model import refit_model
from .select_the_best import select_the_best
from .create_feature_names import create_feature_names
from .multi_output_classifier2 import MultiOutputClassifier2
from .transformer_custom_settings import transformer_custom_settings
from .transformer_default_settings import TransformerDefaultSettings
from .multi_binary_to_multi_class import MultiBinaryToMultiClass
from .proba_transformer import ProbaTransformer



from sklearn.base import BaseEstimator, TransformerMixin
import numpy as np


def lookup(h, b, X, n_bins):
    idx_raw = np.searchsorted(b, X)
    idx_adj = np.minimum(idx_raw - 1, n_bins - 1)
    return h[idx_adj]


class HistDensity(BaseEstimator, TransformerMixin):
    def __init__(self, bins=512):
        self.n_bins = bins

    def fit(self, X, y=None):
        # declare variables
        n_features = X.shape[1]
        self.hist_density_ = np.empty(shape=(self.n_bins, n_features))
        self.bin_edges_ = np.empty(shape=(self.n_bins + 1, n_features))
        # compute histogram for each feature
        for j in range(n_features):
            self.hist_density_[:, j], self.bin_edges_[:, j] = np.histogram(
                X[~np.isnan(X[:, j]), j], bins=self.n_bins, density=True)
        return self

    def transform(self, X, y=None):
        # declare variables
        n_features = self.hist_density_.shape[1]
        output = np.empty(shape=X.shape)
        # lookup densities from fitted histograms
        # getidx=lambda i: min(i - 1, self.n_bins - 1)
        # lookup=lambda h, b, X: [h[getidx(np.searchsorted(b,xi))] for xi in X]
        for j in range(n_features):
            output[:, j] = lookup(
                self.hist_density_[:, j],  # h
                self.bin_edges_[:, j],  # b
                X[:, j],  # X
                self.n_bins)
        return output


trans = HistDensity()

meta = {
    'id': 'pdf1',
    'name': 'Prob Density',
    'description': (
        "Probability density estimated by a histogram. "
        "Lookup density based on fitted bin edges"),
    'keywords': [
        'histogram', 'probability density'],
    'feature_names_prefix': 'pdf_hist'
}

"""Example

from verto.pdf1 import trans, meta
from datasets.demo1 import X_train, Y_train
from seasalt import create_feature_names
import pandas as pd
import numpy as np

trans.set_params(**{'bins': 128})
X_new = trans.fit_transform(X_train)
names = create_feature_names(meta['feature_names_prefix'], X_new.shape[1])
df = pd.DataFrame(data=X_new, columns=names)
df
"""



# Breast Cancer Wisconsin
from sklearn.datasets import load_diabetes
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import minmax_scale
import numpy as np

tmp = load_diabetes()

X_train, X_valid, Y_train, Y_valid = train_test_split(
    tmp.data,
    tmp.target,
    test_size = 0.30,
    random_state = 42
)
del tmp

# generate random folds
fold_ids = np.random.randint(low=1, high=10, size=(len(Y_train),))

# meta data
meta = {
    "id": "demo1",
    "name": "Diabetes datasets",
    "descriptions": (
        "Demo dataset from sklearn. Single ratio-scale output with integer "
        "values between 25 and 365. All inputs has been transformed to "
        "standard-normal interval-scaled features."),
    "source": (
        "https://scikit-learn.org/stable/modules/generated/"
        "sklearn.datasets.load_diabetes.html"),
    "keywords": [
        "demo dataset", "multivariate regression", "interval-scale features"],
    "sample_size": 442,
    "sample_train": len(Y_train),
    "sample_valid": len(Y_valid),
    'output_num': 1,  # 'single'
    'output_scale': 'ratio',
    'output_dtype': 'integer',
    'input_num': 10,  # 'multi'
    'input_scale': 'interval',
    'input_dtype': 'float'
}



from sklearn.base import BaseEstimator, TransformerMixin
from itertools import product
import numpy as np


class MultiBinaryToMultiClass(BaseEstimator, TransformerMixin):
    def __init__(self):
        pass

    def fit(self, Y):
        _, self.n_cols = Y.shape
        self.mapping = list(product([0, 1], repeat=self.n_cols))
        return self

    def transform(self, Y):
        n_rows, _ = Y.shape
        out = np.empty(shape=(n_rows,))
        out[:] = np.nan
        for enc in range(len(self.mapping)):
            idx = np.all(Y == self.mapping[enc], axis=1)
            out[idx] = enc
        return out.astype(int)

    def inverse_transform(self, y):
        out = []
        for enc in y:
            out.append(self.mapping[int(enc)])
        return np.vstack(out)




from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler
from sklearn.linear_model import LassoLarsIC


model = Pipeline(steps=[
    ('scl', StandardScaler()),
    ('lin', LassoLarsIC(
        fit_intercept=True,
        positive = False,
        precompute = True,
        eps = 2.220446049250313e-16,  # for ill-conditioned cholesky
        max_iter = 500,
        normalize=True,
        copy_X = True,
        verbose = False,  # for debugging
    ))
])

hyper = {
    'lin__criterion': ['aic', 'bic'],
}

meta = {
    'id': "simi12",
    'name': 'LinReg Lasso IC',
    'descriptions': (
        "Lasso Regression (L1 penalty), LARS solver, Autoselect l1 penalty "
        "based on information criteria aic or bic"),
    'solver': 'Least Angle Regression (LARS)',
    'active': True,
    'keywords': [
        'linear regression', 'univariate regression', 'multiple regression',
        'model selection', 'LassoLarsIC'],
    'output_num': 'single',
    'output_scale': 'interval',
    'output_dtype': 'float',
    'input_num': 'multi',
    'input_scale': 'interval',
    'input_dtype': 'float'
}



from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.naive_bayes import GaussianNB
import numpy as np
from copy import deepcopy


class SimpleGNB(BaseEstimator, TransformerMixin):
    def __init__(self, proba=True, priors=None, var_smoothing=1e-09):
        self.proba = proba
        self.priors = priors
        self.var_smoothing = var_smoothing

    def fit(self, X, y):
        self.n_features = X.shape[1]  # if len(X.shape) >= 2 else 1
        self.n_targets = y.shape[1]  # if len(y.shape) >= 2 else 1
        self.models = []
        for i in range(self.n_targets):
            for j in range(self.n_features):
                tmp = GaussianNB(
                    priors=self.priors, var_smoothing=self.var_smoothing)
                tmp.fit(X=X[:, j].reshape(-1, 1), y=y[:, i].reshape(-1, 1))
                self.models.append(deepcopy(tmp))
        return self

    def transform(self, X, y=None):
        output = np.empty(
            dtype=np.float16, shape=(len(X), self.n_targets * self.n_features))
        for i in range(self.n_targets):
            for j in range(self.n_features):
                idx = i * self.n_features + j
                if self.proba:
                    output[:, idx] = self.models[idx].predict_proba(X)[:, 1]
                else:
                    output[:, idx] = self.models[idx].predict(X)
        return output


trans = SimpleGNB()

meta = {
    'id': 'nb1',
    'name': 'Simple GNBs',
    'description': (
        "Fit each combination of target and feature as a "
        "simple Naive Bayes model, and use the predictions "
        "as features."),
    'keywords': ['GaussianNB', 'naive bayes'],
    'feature_names_prefix': 'nb'
}



from sklearn.base import BaseEstimator, ClassifierMixin
from copy import deepcopy
import numpy as np


class MultiOutputClassifier2(BaseEstimator, ClassifierMixin):
    def __init__(self, estimator):
        self.estimator = estimator

    def fit(self, X, y, **fit_params):
        # copy base estimators
        self.n_targets = y.shape[1]  # if len(y.shape) >= 2 else 1
        self.estimators_ = []
        for i in range(self.n_targets):
            self.estimators_.append(deepcopy(self.estimator))
        # adjust 'eval_set' in fit_params
        tmp_params = []
        for i in range(self.n_targets):
            tmp_params.append(deepcopy(fit_params))
            if 'eval_set' in tmp_params[i]:
                tmp_X = tmp_params[i]['eval_set'][0]
                tmp_y = tmp_params[i]['eval_set'][1]
                tmp_params[i]['eval_set'] = (tmp_X, tmp_y[:, i])
        # loop over each estimator
        for i in range(self.n_targets):
            self.estimators_[i].fit(X, y[:, i], **tmp_params[i])
        # retrive feature importances
        self.feature_importances_targets_ = np.vstack(
            [estim.feature_importances_ for estim in self.estimators_]).T
        self.feature_importances_ = np.sum(
            self.feature_importances_targets_, axis=1)
        return self

    def predict_proba(self, X):
        return [estim.predict_proba(X) for estim in self.estimators_]

    def predict(self, X):
        tmp = [estim.predict(X) for estim in self.estimators_]
        return np.stack(tmp, axis=1)

    def score(self, X, y):
        y_pred = self.predict(X)
        return np.mean(np.all(y == y_pred, axis=1))





def create_feature_names(prefix, n_features):
    """Generate numbered feature names

    Example
    -------
        from seasalt import create_feature_names
        from verto.dim1 import trans, meta
        from datasets.demo1 import X_train

        trans.set_params(**{'pca__n_components': 3})
        X_new = trans.fit_transform(X_train)
        names = create_feature_names(
            meta['feature_names_prefix'], X_new.shape[1])
        print(names)

        import pandas as pd
        df = pd.DataFrame(data=X_new, columns=names)
        df
    """
    return [prefix + "_" + str(i) for i in range(n_features)]




from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler
from sklearn.linear_model import SGDClassifier
import scipy.stats as ss

model = Pipeline(steps=[
    ('scl', StandardScaler()),
    ('lin', SGDClassifier(
        # Logistic Regression
        loss = 'log',
        penalty = 'l2',
        l1_ratio = 0,
        fit_intercept = True,
        class_weight = {0: 1, 1: 1},  # enforce binary [0,1]
        # solver settings
        max_iter = 1000,
        tol = 1e-3,
        shuffle = True,
        random_state = 42,
        # adaptive learning
        learning_rate = 'adaptive',
        eta0 = 0.5,
        # early stopping
        early_stopping = True,
        validation_fraction = 0.15,
        n_iter_no_change = 10,
        # other
        warm_start = True,
        average = 20,  # after 20 epochs, compute average weights
    ))
])

hyper = {
    # 'lin__alpha': ss.expon(loc=1e-8, scale=.1),
    # 'lin__alpha': ss.uniform(1e-4, 1.1),   # C ~ [1/1.1, 10000]
    # 'lin__alpha': ss.gamma(a=2.2, loc=1e-5, scale=.7),  # alpha~[0.004, 9.8]
    'lin__alpha': ss.gamma(a=1.5, loc=1e-5, scale=.7),  # alpha ~ [0.001, 10]
}

meta = {
    'id': "sbmi60",
    'name': 'Logistic Ridge',
    'descriptions': (
        "Logistic Regression, L2 penalty (Ridge), SGD solver, standard-normal "
        "transformed features."),
    'solver': 'Stochastic Gradient Descent (SGD)',
    'active': True,
    'keywords': [
        'binary classification', 'linear regression', 'SGDClassifier'],
    'output_num': 'single',
    'output_scale': 'binary',
    'output_dtype': 'bool',
    'input_num': 'multi',
    'input_scale': 'interval',
    'input_dtype': 'float'
}



from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler
from sklearn.decomposition import PCA


trans = Pipeline(steps=[
    ('scl', StandardScaler(with_mean=True, with_std=True, copy=True)),
    ('pca', PCA(
        n_components='mle', svd_solver='full',
        whiten=False, copy=True))
])

meta = {
    'id': 'dim2',
    'name': 'PCA Minka MLE',
    'description': "Minka's MLE to guess n_components",
    'keywords': [
        'dimensionality reduction', 'principal component anlysis',
        'StandardScaler', 'PCA', 'Minka MLE'],
    'feature_names_prefix': 'dim_minka'
}


"""Example

from verto.dim2 import trans, meta
from datasets.demo1 import X_train
from seasalt import create_feature_names
import pandas as pd
import numpy as np

X_new = trans.fit_transform(X_train).astype(np.float16)
names = create_feature_names(meta['feature_names_prefix'], X_new.shape[1])
df = pd.DataFrame(data=X_new, columns=names)
df
"""




from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler
from sklearn.linear_model import LogisticRegression
import scipy.stats as ss

model = Pipeline(steps=[
    ('scl', StandardScaler()),
    ('lin', LogisticRegression(
        penalty = 'l2',
        solver = 'newton-cg',
        multi_class = 'ovr',   # binary
        random_state = 42,
        fit_intercept = True,
        max_iter = 100,
        warm_start = True,  # reuse previous solution
    ))
])

hyper = {
    'lin__C': ss.gamma(a=2.5, loc=1e-5, scale=0.75),
    # 'lin__fit_intercept': [True, False]
}

meta = {
    'id': "sbmi59",
    'name': 'Logistic Ridge',
    'descriptions': 'Logistic Regression, L2 penalty (Ridge)',
    'solver': 'Newton-CG',
    'active': True,
    'keywords': ['binary classification', 'linear regression'],
    'output_num': 'single',
    'output_scale': 'binary',
    'output_dtype': 'bool',
    'input_num': 'multi',
    'input_scale': 'interval',
    'input_dtype': 'float'
}



# Interpret CV results - Pick the Best
#    n_best  depends on cv_settings['n_iter']
#    lowest=False  means that max(score) is the "best"
import pandas as pd


def select_the_best(res, n_best=5, lowest=False):
    # copy into dataframe
    res = pd.DataFrame(res.cv_results_)

    # pick the N best
    res.sort_values(by='mean_test_score', ascending=lowest, inplace=True)
    res = res[:n_best]

    # compute CV-ratio depending on the score type
    if lowest:  # small mean(cv), small std(cv)
        res['cvratio'] = -(res['mean_test_score'] * res['std_test_score'])
    else:  # high mean(cv), small std(cv)
        res['cvratio'] = res['mean_test_score'] / res['std_test_score']

    # pick the best CV-Ratio of the N best ("best" must be the highest)
    res.sort_values(by='cvratio', ascending=False, inplace=True)

    # prep output args
    bestparams = res['params'][res.index[0]]  # best params

    # Create nice summary
    summary = pd.DataFrame.from_dict(list(res['params'].values))
    summary.index = res.index
    col_names = [
        'cvratio', 'rank_test_score', 'mean_test_score',
        'std_test_score', 'mean_fit_time', 'std_fit_time']
    summary[col_names] = res[col_names]

    # done
    return bestparams, summary



from sklearn.preprocessing import QuantileTransformer


trans = QuantileTransformer(
    n_quantiles = 512,
    output_distribution = 'uniform'  # uniform, normal
)

meta = {
    'id': 'scl1',
    'name': 'Quantile Scaler',
    'description': (
        "Transform data to by its percentiles to "
        "an uniform distribution"),
    'keywords': [
        'QuantileTransformer', 'probability density'],
    'feature_names_prefix': 'scl_qntl'
}



from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler
from sklearn.decomposition import PCA
import numpy as np
import statsmodels.api as sm


class PcaModelSelect(Pipeline):
    """Principal component analysis (PCA) wheras the number
    of components by simple backtracking line search and
    model selection criteria

    Parameters
    ----------
    score : str (Default: 'bic')
        The model selection criteria to minimze
        - 'bic' (default)
        - 'aic'
        - 'prsquared'

    tol : float (Default: 1e-3)
        Required percentage decrease in each iteration.

    max_iter: int (Default: None)
        Any number between 1 and the number of features

    verbose: bool (Default: False)

    Algorithm
    ---------
        1. Increment the number of components by one
        2. Run the PCA
        3. Fit a Lasso-Logit model with the components
        4. Check if model selection criteria improves
        5. If not, terminate backtracking; Else got to 1
    """
    def __init__(self, score='bic', tol=1e-3, verbose=False, max_iter=None,
                 alpha=1.0):
        self.score = score
        self.tol = tol
        self.verbose = verbose
        self.max_iter = max_iter
        self.alpha = alpha
        super().__init__(steps=[
            ('scl', StandardScaler(with_mean=True, with_std=True, copy=True)),
            ('pca', PCA(
                n_components=1,
                svd_solver='full', whiten=False, copy=True))
        ])

    def transform(self, X, y=None):
        return super().transform(X).astype(np.float16)

    def fit(self, X, y):
        # start values
        prev = None
        redu = 1.0
        status = 'proceed'

        # max n_components
        if self.max_iter is None:
            self.max_iter = X.shape[1]
        max_iter = min(self.max_iter, X.shape[1])

        # display
        if self.verbose:
            print("{:3s} | {:8s} | {:8s} | {}".format(
                'num', 'score', 'improved', 'status'))

        # backtracking loop
        for n_comp in range(1, max_iter):
            # set PCA parameter
            super().set_params(**{'pca__n_components': n_comp})

            # compute components
            X_new = super().fit_transform(X)

            # run Logistic Regression
            lr = sm.Logit(y, sm.add_constant(X_new)).fit_regularized(
                method='l1', alpha=self.alpha, disp=0)

            # read current score (target fun value to minimize)
            curr = getattr(lr, self.score)
            if self.score == 'prsquared':
                curr = 1 - curr

            # minimization task
            if prev is None:
                prev = curr  # First iteration
            else:
                redu = 1 - curr / prev
                prev = curr
                if redu > self.tol:
                    self.best_n_components_ = n_comp
                    self.best_statsmodel_ = lr
                else:
                    status = 'terminated'
                    break

            # display - progress
            if self.verbose:
                print("{:3d} | {:8.4f} | {:8.4f} | {}".format(
                    n_comp, curr, redu, status))

        # display - final resul
        if self.verbose:
            print(self.best_statsmodel_.summary())

        # reset best solution
        super().set_params(**{'pca__n_components': self.best_n_components_})

    def fit_transform(self, X, y):
        self.fit(X, y)
        return self.transform(X)


trans = PcaModelSelect(score='bic', tol=1e-3)

meta = {
    'id': 'dim4',
    'name': 'PCA BIC',
    'description': (
        "Number of components is determined by fitting a Lasso-Logit "
        "model and minimize the BIC criteron."),
    'keywords': [
        'dimensionality reduction', 'principal component anlysis',
        'StandardScaler', 'PCA', 'BIC', 'Logistic Regression',
        'statsmodels'],
    'feature_names_prefix': 'dim_bic'
}

"""Example

from verto.dim4 import trans, meta
from datasets.demo1 import X_train, Y_train
from seasalt import create_feature_names
import pandas as pd
import numpy as np

#trans.set_params(**{'verbose': True, 'tol': 1e-3})
X_new = trans.fit_transform(X_train, Y_train)
names = create_feature_names(meta['feature_names_prefix'], X_new.shape[1])
df = pd.DataFrame(data=X_new, columns=names)
df
"""



from setuptools import setup
# from setuptools import setup, find_packages


def read(fname):
    import os
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='potpourri',
    version='0.13.1',
    description='model zoo of different preconfigured algorithms',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    url='http://github.com/kmedian/potpourri',
    author='Ulf Hamster',
    author_email='554c46@gmail.com',
    license='MIT',
    packages=['seasalt', 'datasets', 'verto', 'potpourri'],
    # packages=find_packages(),
    install_requires=[
        'setuptools>=40.0.0',
        'nose>=1.3.7',
        'numpy>=1.15.3',
        'scipy>=1.1.0',
        'pandas>=0.23.4',
        'korr>=0.6.0',
        'scikit-learn>=0.20.0',
        'xgboost>=0.80',
        'catboost>=0.11.1',
        'lightgbm>=2.2.2',
        'sklearn-lvq>=1.1.0',
        'umap-learn>=0.3.7',
        'MulticoreTSNE>=0.0.1.1',
        'mlxtend>=0.14.0'],
    python_requires='>=3.6',
    zip_safe=False
)



from mlxtend.preprocessing import CopyTransformer

trans = CopyTransformer()

meta = {
    'id': 'raw1',
    'name': 'Raw Features',
    'description': (
        "Just copy raw features to comply with "
        "this preprocessing framewortk."),
    'keywords': ['CopyTransformer', 'mlxtend'],
    'feature_names_prefix': 'raw'
}



from sklearn.base import BaseEstimator, TransformerMixin
import numpy as np
import itertools as it
from seasalt import create_feature_names


class TransformerDefaultSettings(BaseEstimator, TransformerMixin):
    def __init__(self, module_list):
        self.module_list = module_list
        # import modules
        self.modules_ = []
        for m in self.module_list:
            self.modules_.append(__import__(m, fromlist=['trans', 'meta']))

    def fit(self, X, y=None):
        # fit modules
        for i in range(len(self.modules_)):
            self.modules_[i].trans.fit(X, y)
        return self

    def transform(self, X, y=None):
        Z = []
        for i in range(len(self.modules_)):
            Z.append(self.modules_[i].trans.transform(X))

        # create names if not exist
        if not hasattr(self, 'feature_names_'):
            f = []
            for i in range(len(self.modules_)):
                f.append(create_feature_names(
                    self.modules_[i].meta['feature_names_prefix'],
                    Z[i].shape[1]))
            self.feature_names_ = list(it.chain.from_iterable(f))

        return np.concatenate(Z, axis=1)


"""
Example

trn = TransformerDefaultSettings(["verto.raw1", "verto.rnd1"])
trn.fit(X_train, Y_train)
X_new = trn.transform(X_train)
pd.DataFrame(data=X_new, columns=trn.feature_names_)
"""



from sklearn.base import BaseEstimator, TransformerMixin
import numpy as np


class RandomFeature(BaseEstimator, TransformerMixin):
    def __init__(self, n_cols=1):
        self.n_cols = n_cols

    def fit(self, X, y=None):
        return self

    def transform(self, X, y=None):
        n_rows = len(X)
        return np.random.standard_normal((n_rows, self.n_cols))


trans = RandomFeature()

meta = {
    'id': 'rnd1',
    'name': 'Random Feature',
    'description': (
        "Generate 1 or more random standard normal variable(s). "
        "Use it in pre-analysing feature importances."),
    'keywords': ['standard_normal'],
    'feature_names_prefix': 'rnd_std'
}



from sklearn.base import BaseEstimator, TransformerMixin


class ProbaTransformer(BaseEstimator, TransformerMixin):
    def __init__(self, model, drop=True):
        self.model = model
        self.drop = drop

    def fit(self, *args, **kwargs):
        self.model.fit(*args, **kwargs)
        return self

    def transform(self, X, **transform_params):
        Xtrans = self.model.predict_proba(X)
        if self.drop:
            Xtrans = Xtrans[:, 1:]
        return Xtrans




from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler
from sklearn.linear_model import LogisticRegression
import scipy.stats as ss

model = Pipeline(steps=[
    ('scl', StandardScaler()),
    ('lin', LogisticRegression(
        penalty = 'l1',
        solver = 'liblinear',  # for L1
        multi_class = 'ovr',  # binary
        random_state = 42,
        fit_intercept = True,
        max_iter = 100,
    ))
])

hyper = {
    'lin__C': ss.gamma(a=2.2, loc=1e-3, scale=0.7),
    'lin__intercept_scaling': ss.uniform(.1, 1.9),
    # 'lin__fit_intercept': [True, False]
}

meta = {
    'id': "sbmi56",
    'name': 'Logistic Lasso',
    'descriptions': 'Logistic Regression with L1 penalty (Lasso)',
    'solver': 'liblinear',
    'active': True,
    'keywords': ['binary classification', 'linear regression'],
    'output_num': 'single',
    'output_scale': 'binary',
    'output_dtype': 'bool',
    'input_num': 'multi',
    'input_scale': 'interval',
    'input_dtype': 'float'
}
